==> aexpy-0.1.0.tar.gz: aexpy-0.1.0/aexpy/third/pycompat/differ.py <==
import code
import pathlib
import subprocess
from datetime import datetime, timedelta
from logging import Logger
from pathlib import Path
from typing import Callable
from uuid import uuid1

from aexpy import getCacheDirectory, json
from aexpy.differing.checkers import DiffRule, diffrule, fortype
from aexpy.differing.default import RuleDiffer
from aexpy.environments.conda import CondaEnvironment
from aexpy.evaluating.default import Evaluator as BaseEvaluator
from aexpy.extracting.environments import (EnvirontmentExtractor,
                                           ExecutionEnvironment)
from aexpy.models import (ApiBreaking, ApiDescription, ApiDifference,
                          Distribution, Release, Report)
from aexpy.models.description import FunctionEntry
from aexpy.models.difference import BreakingRank, DiffEntry
from aexpy.pipelines import Pipeline
from aexpy.preprocessing import getDefault
from aexpy.producer import ProducerOptions
from aexpy.reporting import Reporter as Base


@fortype(FunctionEntry)
@diffrule
def AddRequiredParameter(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pb) - set(pa)

    items = []

    for item in new:
        if not b.getParameter(item).optional:
            items.append(item)
    return [DiffEntry(message=f"Add required parameter ({a.id}): {item}") for item in items]


@fortype(FunctionEntry)
@diffrule
def RemoveRequiredParameter(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pa) - set(pb)

    items = []

    for item in new:
        if not a.getParameter(item).optional:
            items.append(item)

    return [DiffEntry(message=f"Remove required parameter ({a.id}): {item}") for item in items]


@fortype(FunctionEntry)
@diffrule
def AddOptionalParameter(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pb) - set(pa)

    items = []

    for item in new:
        if b.getParameter(item).optional:
            items.append(item)

    return [DiffEntry(message=f"Add optional parameter ({a.id}): {item}") for item in items]


@fortype(FunctionEntry)
@diffrule
def RemoveOptionalParameter(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pa) - set(pb)

    items = []

    for item in new:
        if a.getParameter(item).optional:
            items.append(item)

    return [DiffEntry(message=f"Remove optional parameter ({a.id}): {item}") for item in items]


@fortype(FunctionEntry)
@diffrule
def ReorderParameter(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pa) & set(pb)

    items = []

    for item in new:
        for titem in new:
            if item == titem:
                continue
            if pa.index(item) < pa.index(titem) and pb.index(item) > pb.index(titem):
                items.append((item, titem))

    return [DiffEntry(message=f"Reorder parameter ({a.id}): {item} & {titem}") for item, titem in items]


@fortype(FunctionEntry)
@diffrule
def AddParameterDefault(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pa) & set(pb)

    items = []

    for item in new:
        if not a.getParameter(item).optional and b.getParameter(item).optional:
            items.append((item, b.getParameter(item).default))

    return [DiffEntry(message=f"Add parameter default ({a.id}): {item} = {value}") for item, value in items]


@fortype(FunctionEntry)
@diffrule
def RemoveParameterDefault(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pa) & set(pb)

    items = []

    for item in new:
        if a.getParameter(item).optional and not b.getParameter(item).optional:
            items.append((item, a.getParameter(item).default))

    return [DiffEntry(message=f"Remove parameter default ({a.id}): {item} = {value}") for item, value in items]


@fortype(FunctionEntry)
@diffrule
def ChangeParameterDefault(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pa) & set(pb)

    items = []

    for item in new:
        if a.getParameter(item).optional and b.getParameter(item).optional:
            da = a.getParameter(item).default
            db = b.getParameter(item).default
            if da != "___complex_type___" and db != "___complex_type___" and da != db:
                items.append((item, da, db))

    return [DiffEntry(message=f"Change parameter default ({a.id}): {item} = {da} -> {db}") for item, da, db in items]


class Differ(RuleDiffer):
    def defaultCache(self) -> "Path | None":
        return super().defaultCache() / "pycompat"

    def __init__(self, logger: "Logger | None" = None, cache: "Path | None" = None, options: "ProducerOptions | None" = None, rules: "list[DiffRule] | None" = None) -> None:
        rules = rules or []

        from aexpy.differing.rules import (aliases, attributes, classes,
                                           externals, functions, modules,
                                           parameters, types)

        rules.append(classes.AddClass)
        rules.append(classes.RemoveClass)
        rules.append(functions.AddFunction)
        rules.append(functions.RemoveFunction)
        rules.append(AddRequiredParameter)
        rules.append(RemoveRequiredParameter)
        rules.append(ReorderParameter)
        rules.append(AddOptionalParameter)
        rules.append(RemoveOptionalParameter)
        rules.append(AddParameterDefault)
        rules.append(RemoveParameterDefault)
        rules.append(ChangeParameterDefault)
        rules.append(attributes.AddAttribute)
        rules.append(attributes.RemoveAttribute)

        super().__init__(logger, cache, options, rules)

==> aexpy-0.1.1.tar.gz: aexpy-0.1.1/aexpy/third/pycompat/differ.py <==
import code
import pathlib
import subprocess
from datetime import datetime, timedelta
from logging import Logger
from pathlib import Path
from typing import Callable
from uuid import uuid1

from aexpy import json
from aexpy.diffing import Differ
from aexpy.diffing.differs.checkers import DiffConstraint, diffcons, fortype
from aexpy.diffing.differs.default import ConstraintDiffer
from aexpy.diffing.evaluators.checkers import EvalRule
from aexpy.diffing.evaluators.default import RuleEvaluator
from aexpy.environments.conda import CondaEnvironment
from aexpy.extracting.environments import (EnvirontmentExtractor,
                                           ExecutionEnvironment)
from aexpy.models import (ApiDescription, ApiDifference,
                          Distribution, Release, Report)
from aexpy.models.description import FunctionEntry
from aexpy.models.difference import BreakingRank, DiffEntry
from aexpy.pipelines import Pipeline
from aexpy.producers import ProducerOptions
from aexpy.reporting import Reporter as Base


@fortype(FunctionEntry)
@diffcons
def AddRequiredParameter(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pb) - set(pa)

    items = []

    for item in new:
        if not b.getParameter(item).optional:
            items.append(item)
    return [DiffEntry(message=f"Add required parameter ({a.id}): {item}") for item in items]


@fortype(FunctionEntry)
@diffcons
def RemoveRequiredParameter(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pa) - set(pb)

    items = []

    for item in new:
        if not a.getParameter(item).optional:
            items.append(item)

    return [DiffEntry(message=f"Remove required parameter ({a.id}): {item}") for item in items]


@fortype(FunctionEntry)
@diffcons
def AddOptionalParameter(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pb) - set(pa)

    items = []

    for item in new:
        if b.getParameter(item).optional:
            items.append(item)

    return [DiffEntry(message=f"Add optional parameter ({a.id}): {item}") for item in items]


@fortype(FunctionEntry)
@diffcons
def RemoveOptionalParameter(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pa) - set(pb)

    items = []

    for item in new:
        if a.getParameter(item).optional:
            items.append(item)

    return [DiffEntry(message=f"Remove optional parameter ({a.id}): {item}") for item in items]


@fortype(FunctionEntry)
@diffcons
def ReorderParameter(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pa) & set(pb)

    items = []

    for item in new:
        for titem in new:
            if item == titem:
                continue
            if pa.index(item) < pa.index(titem) and pb.index(item) > pb.index(titem):
                items.append((item, titem))

    return [DiffEntry(message=f"Reorder parameter ({a.id}): {item} & {titem}") for item, titem in items]


@fortype(FunctionEntry)
@diffcons
def AddParameterDefault(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pa) & set(pb)

    items = []

    for item in new:
        if not a.getParameter(item).optional and b.getParameter(item).optional:
            items.append((item, b.getParameter(item).default))

    return [DiffEntry(message=f"Add parameter default ({a.id}): {item} = {value}") for item, value in items]


@fortype(FunctionEntry)
@diffcons
def RemoveParameterDefault(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pa) & set(pb)

    items = []

    for item in new:
        if a.getParameter(item).optional and not b.getParameter(item).optional:
            items.append((item, a.getParameter(item).default))

    return [DiffEntry(message=f"Remove parameter default ({a.id}): {item} = {value}") for item, value in items]


@fortype(FunctionEntry)
@diffcons
def ChangeParameterDefault(a: "FunctionEntry", b: "FunctionEntry", **kwargs):
    pa = [p.name for p in a.parameters]
    pb = [p.name for p in b.parameters]

    new = set(pa) & set(pb)

    items = []

    for item in new:
        if a.getParameter(item).optional and b.getParameter(item).optional:
            da = a.getParameter(item).default
            db = b.getParameter(item).default
            if da != "___complex_type___" and db != "___complex_type___" and da != db:
                items.append((item, da, db))

    return [DiffEntry(message=f"Change parameter default ({a.id}): {item} = {da} -> {db}") for item, da, db in items]


class PDiffer(ConstraintDiffer):
    def __init__(self, logger: "Logger | None" = None, constraints: "list[DiffConstraint] | None" = None) -> None:
        constraints = constraints or []

        from aexpy.diffing.differs.contraints import (aliases, attributes, classes,
                                                      externals, functions, modules,
                                                      parameters, types)

        constraints.append(classes.AddClass)
        constraints.append(classes.RemoveClass)
        constraints.append(functions.AddFunction)
        constraints.append(functions.RemoveFunction)
        constraints.append(AddRequiredParameter)
        constraints.append(RemoveRequiredParameter)
        constraints.append(ReorderParameter)
        constraints.append(AddOptionalParameter)
        constraints.append(RemoveOptionalParameter)
        constraints.append(AddParameterDefault)
        constraints.append(RemoveParameterDefault)
        constraints.append(ChangeParameterDefault)
        constraints.append(attributes.AddAttribute)
        constraints.append(attributes.RemoveAttribute)

        super().__init__(logger, constraints)


class PEvaluator(RuleEvaluator):
    def __init__(self, logger: "Logger | None" = None, rules: "list[EvalRule] | None" = None) -> None:
        rules = rules or []

        from aexpy.diffing.evaluators import evals
        from aexpy.diffing.evaluators.checkers import rankAt

        rules.append(evals.AddClass)
        rules.append(evals.RemoveClass)
        rules.append(evals.AddFunction)
        rules.append(evals.RemoveFunction)
        rules.append(rankAt("AddRequiredParameter", BreakingRank.High))
        rules.append(rankAt("RemoveRequiredParameter", BreakingRank.High))
        rules.append(rankAt("ReorderParameter", BreakingRank.High))
        rules.append(rankAt("AddOptionalParameter", BreakingRank.High))
        rules.append(rankAt("RemoveOptionalParameter", BreakingRank.High))
        rules.append(rankAt("AddParameterDefault", BreakingRank.High))
        rules.append(rankAt("RemoveParameterDefault", BreakingRank.High))
        rules.append(rankAt("ChangeParameterDefault", BreakingRank.High))
        rules.append(evals.AddAttribute)
        rules.append(evals.RemoveAttribute)

        super().__init__(logger, rules)


class CombinedDiffer(Differ):
    def diff(self, old: "ApiDescription", new: "ApiDescription", product: "ApiDifference"):
        PDiffer(self.logger).diff(old, new, product)
        PEvaluator(self.logger).diff(old, new, product)
